#! /bin/sh
#
#  Authors:
#    Denys Duchier (duchier@ps.uni-sb.de)
# 
#  Contributors:
#    Michael Mehl (mehl@dfki.de)
#    Ralf Scheidhauer (scheidhr@ps.uni-sb.de)
# 
#  Copyright:
#    Organization or Person (Year(s))
# 
#  Last change:
#    $Date: 1999/11/29 10:25:11 $ by $Author: kornstae $
#    $Revision: 1.20 $
# 
#  This file is part of Mozart, an implementation 
#  of Oz 3:
#     http://www.mozart-oz.org
# 
#  See the file "LICENSE" or
#     http://www.mozart-oz.org/LICENSE.html
#  for information on usage and redistribution 
#  of this file, and for a DISCLAIMER OF ALL 
#  WARRANTIES.
#
#

#
# oztool c++ ...        invokes the right c++ compiler with the right options
# oztool cc  ...        same for the c compiler
# oztool ld  ...        creates DLLs
# oztool platform       echoes the oz platform

# determine oztool includes
if test "${OZTOOL_INCLUDES-NONE}" = NONE; then
  if test -n "$OZHOME" && test -r "$OZHOME/include/mozart.h"; then
    OZTOOL_INCLUDES="-I$OZHOME/include"
  else
    oz_tmp=`dirname $0`
    oz_tmp=`cd $oz_tmp/.. & pwd`
    if test -r "$oz_tmp/include/mozart.h"; then
      OZTOOL_INCLUDES="-I$oz_tmp/include"
    else
      OZTOOL_INCLUDES="-I/home/mozart/httpd/html/build/build-1.1.0/install/include";
    fi
  fi
fi

: ${oztool_cxx="c++  -fno-exceptions -fno-rtti  -fpic $OZTOOL_INCLUDES"}
: ${oztool_cc="gcc  -fpic $OZTOOL_INCLUDES"}
: ${oztool_ld="gcc -shared -L/home/mozart/httpd/html/build/build-1.1.0/packages/linux-i486/lib -s"}
: ${oztool_platform="linux-i486"}

if test $# -eq 0
then
  cat <<EOF
Usage:
	oztool c++ -c SourceFile
	oztool cc  -c SourceFile
	oztool ld  -o TargetLib FileList
	oztool platform
EOF
  exit 2
fi

cmd=$1
shift

case "$cmd" in
  c++) cmd=$oztool_cxx;;
  cc ) cmd=$oztool_cc;;
  ld ) cmd=$oztool_ld;;
  platform )
    echo "$oztool_platform"
    exit 0;;
  *  )
  echo "oztool: unknown command $cmd" 1>&2
  exit 1
  ;;
esac

exec $cmd "$@"



